; ----------------------------------------------------------------------------
;
; AutoIt Version: 3.1.1.124 (beta)
; Author:         Thorsten Duhn <Thorsten.Duhn@editorial.de>
;
; Script Function:
;	Create M3U playlist from drag&drop
;
; Script Version: $scriptVersion
;
; ----------------------------------------------------------------------------
#include <Array.au3>
#include <GUIConstants.au3>

Global $scriptName = "CreateM3U"
Global $scriptVersion = "0.1.1"
Global $scriptURL = "http://lostfound.raum108.de/create_m3u/"
Global $scriptHelp = "Create .m3u playlists with drag&drop of files and folders.\n"
$scriptHelp &= "For each folder a single .m3u file is created named after this folder.\n"
$scriptHelp &= "It will not replace already existing playlists of that name.\n\n"
$scriptHelp &= "Usage: Drop media files and folders into edit area.\n"
$scriptHelp &= "Limit: about 32 kbyte file/folder names each call.\n\n"
$scriptHelp &= "Also possible is to drop media files and folders onto program icon.\n"
$scriptHelp &= "Limits: 64 files/folders or\n"
$scriptHelp &= "    4 kbyte file/folder names each call on XP or later, 2 kbyte else.\n\n"
$scriptHelp &= $scriptURL & "\n"
$scriptHelp = StringReplace($scriptHelp,"\n",@CRLF) 

$iniFile = @ScriptDir&"\"&$scriptName&".ini"
If FileExists($iniFile) == 0 Then
   errorMsg("INI file cannot be found.")
   Exit(1)
EndIf
Global $mediaFilter = IniRead($iniFile,"config","mediaFilter","mp3 wav oma mp2 aac ogg m4a mid midi aif aiff wma")
Global $listExt = IniRead($iniFile,"config","listExt","m3u")
Global $recurseDirectories = IniRead($iniFile,"config","recurseDirectories",1)

; GUI position is stored per user in registry
Global $scriptRegKey = "HKEY_CURRENT_USER\Software\" & $scriptName
Global $GUIPosLeft = RegRead($scriptRegKey,"GUIPosLeft")
If @error <> 0 Then $GUIPosLeft = "-1"
Global $GUIPosTop = RegRead($scriptRegKey,"GUIPosTop")
If @error <> 0 Then $GUIPosTop = "-1"

If $CmdLine[0] = "0" Then
   While 1
      $tmp = getFileListByDropBox()
      If $tmp[0] <> "0" Then
         _GUIProgressCreate()
         handleFileArray($tmp)
         _GUIProgressDelete()
      Else
         ExitLoop
      EndIf
   WEnd
Else
   ; ??? check cmd param limitation ???
   _GUIProgressCreate()
   handleFileArray($CmdLine)
   _GUIProgressDelete()
EndIf

; write current GUI position to registry
If $GUIPosLeft <> -1 Then
   RegWrite($scriptRegKey,"GUIPosLeft","REG_SZ",$GUIPosLeft)
   RegWrite($scriptRegKey,"GUIPosTop","REG_SZ",$GUIPosTop)
EndIf

Exit(0)

; *** END of script ***

Func _GUIProgressCreate()
   ; shows progress bar
   GUICreate($scriptName,180,60,-1,-1,$DS_MODALFRAME)
   GUICtrlSetState(GUICtrlCreateAvi(@ScriptDir&"\res\progress.avi",-1,78,5),1)
   GuiCtrlCreateLabel("In progress", 12, 10)
   GUISetState()
EndFunc

Func _GUIProgressDelete()
   GUIDelete()
EndFunc

Func handleFileArray($parameterArray)
   Local $filesArray[1]
   For $i = 1 To $parameterArray[0]
      $currentParameter = StringReplace($parameterArray[$i],"/","\")
      If isDirectory($currentParameter) = 1 Then
         exploreDirectory($currentParameter)
      Else
         _ArrayAdd($filesArray,$currentParameter)
      EndIf
   Next
   ; handle D&D files (may come from different folders)
   _ArrayDelete($filesArray,0)
   If $filesArray[0] <> "0" Then
      _ArraySort($filesArray,0)
      $filesArray = filterFileArray($filesArray,$mediaFilter)
      Local $fileListArray[1]
      For $i = 1 To UBound($filesArray)
         $filePath =$filesArray[$i - 1]
         $currentParameter = getPathName($filePath)
         If ($i = 1) Or ($formerFolder = $currentParameter) Then
            _ArrayAdd($fileListArray,getFileName($filePath))
         Else
            _ArrayDelete($fileListArray,0)
            createFileList($formerFolder,$fileListArray)
            ReDim $fileListArray[1]
            _ArrayAdd($fileListArray,getFileName($filePath))
         EndIf
         $formerFolder = $currentParameter
      Next
      _ArrayDelete($fileListArray,0)
      createFileList($formerFolder,$fileListArray)
   EndIf
EndFunc

Func getFileListByDropBox()
   Local $Form1 = GUICreate($scriptName, 345, 175, $GUIPosLeft, $GUIPosTop, -1, $WS_EX_ACCEPTFILES)
   Local $exitMenu = GuiCtrlCreateMenuItem("&Exit",-1)
   Local $aboutMenu = GuiCtrlCreateMenuItem("&About",-1)
   GUICtrlCreateLabel($scriptURL,5,140)
   Local $Edit1 = GUICtrlCreateEdit("", 5, 2, 335, 133)
   GUICtrlSetState($Edit1, $GUI_ACCEPTFILES)
   GUICtrlCreatePic (@ScriptDir&"\res\dropzone.bmp",15,20,0,0)
   GUISetState(@SW_SHOW)
   Local $filesArray[1]
   Local $msg
   Local $dropValue
   While 1
       $msg = GUIGetMsg()
       $dropValue = GUICtrlRead($Edit1)
       Select
           Case $dropValue <> ""
               ExitLoop
           Case $msg = $GUI_EVENT_CLOSE
               ExitLoop
           Case $msg = $exitMenu
               ExitLoop
           Case $msg = $aboutMenu
               aboutMsg()
           Case Else
               ;
       EndSelect
   WEnd

   $tmp = WinGetPos($Form1)
   If UBound($tmp) > 0 Then
      $GUIPosLeft = $tmp[0]
      If $GUIPosLeft = -1 Then $GUIPosLeft = 0 
      $GUIPosTop = $tmp[1]
      If $GUIPosTop = -1 Then $GUIPosTop = 0 
   EndIf

   GUIDelete($Form1)
   $dropValue = StringStripWS(StringStripCR($dropValue),2)
   If $dropValue <> "" Then
      If StringInStr($dropValue,@LF) Then
         $filesArray = StringSplit($dropValue,@LF)
      Else
         _ArrayAdd($filesArray,$dropValue)
         $filesArray[0] = 1
      EndIf
   Else
      $filesArray[0] = 0
   EndIf
   Return $filesArray
EndFunc

Func getPathName($file)
   ; get path without trailing backslash from full filename
   If isDirectory($file) = 1 Then
      return $file
   Else
      return StringLeft($file,StringInStr($file,"\",0,-1) - 1)
   EndIf
EndFunc

Func getFileName($file)
   ; get file name without path
   If isDirectory($file) = 1 Then
      return ""
   Else
      return StringMid($file,StringInStr($file,"\",0,-1) + 1)
   EndIf
EndFunc

Func exploreDirectory($path)
   If isDirectory($path) = 1 Then
      ; restrict recursion to avoid loops because of directory junctions:
      Local $loopLimit = 128
      If (StringStrCount($path,"\") + 1) < $loopLimit Then
         Local $fileList = fileListFiltered($path,$mediaFilter)
         createFileList($path,$fileList)
         If $recurseDirectories = 1 Then
            Local $dirList = dirList($path)
            If UBound($dirList) > 0 Then
               While 1
                  Local $dirName = _ArrayPop($dirList)
                  If $dirName = "" Then ExitLoop
                  exploreDirectory($path & "\" & $dirName)
               WEnd
            EndIf
         EndIf
      Else
         errorMsg("Path contains more than " & ($loopLimit - 1) & " directories. Stopped recursion to avoid directory junction loop." & @CRLF & $path & "\" & $dirName)
      EndIf
   EndIf
EndFunc

Func StringStrCount($string,$substring)
   If StringInStr($string,$substring) = 0 Then
      return 0
   Else
      Local $tmp = StringSplit($string,$substring)
      return $tmp[0] - 1
   EndIf
EndFunc

Func createFileList($folder,$fileList)
   If $folder <> "" Then
      Local $tmp = StringInStr($folder,"\",0,-1) + 1
      Local $listName = $folder & "\" & StringMid($folder,$tmp) & "." & $listExt
      If fileExists($listName) = 0 And $fileList[0] <> "0" Then
         Local $listHandle = FileOpen($listName,2)
         If $listHandle = -1 Then
            errorMsg("Error opening file to write: " & @CRLF & $listName)
            Exit(1)
         EndIf
         If UBound($fileList) > 1 Then
            $tmp = StringAddCR(_ArrayToString($fileList,@LF))
         Else
            $tmp = $fileList[0]
         EndIf 
         FileWrite ($listHandle,$tmp & @CRLF)
         FileClose($listHandle)
      EndIf
   EndIf
EndFunc

Func errorMsg($txt)
   MsgBox(16,$scriptName&" "&$scriptVersion&": Error",$txt)
EndFunc

Func aboutMsg()
   MsgBox(64,$scriptName,$scriptName&" "&$scriptVersion & @CRLF & @CRLF & $scriptHelp)
EndFunc

Func fileList($folder)
   Local $filesArray[1]
   Local $search = FileFindFirstFile($folder & "\*.*")
   Local $fileName
   If $search > -1 Then
      While 1
         $fileName = FileFindNextFile($search) 
         If @error Then ExitLoop
         If isDirectory($folder &"\"& $fileName) = 0 Then
            _ArrayAdd($filesArray,$fileName)
         EndIf
      WEnd
      FileClose($search)
   EndIf
   _ArrayDelete($filesArray,0)
   _ArraySort($filesArray,0)
   return $filesArray
EndFunc

Func fileListFiltered($folder,$extFilter)
   ; extFilter is a space seperated list of file extentions
   return filterFileArray(fileList($folder),$extFilter)
EndFunc

Func filterFileArray($filesArray,$extFilter)
   ; extFilter is a space seperated list of file extentions
   Local $filterArray = StringSplit($extFilter," ")
   _ArrayDelete($filterArray,0)
   _ArraySort($filterArray,0)
   Local $resultArray[1]
   Local $fileName
   Local $tmp
   While 1
      $fileName = _ArrayPop($filesArray)
      If $fileName = "" Then ExitLoop
      $tmp = StringInStr($fileName,".",0,-1) + 1
      If $tmp > 0 And _ArrayBinarySearch($filterArray,StringMid($fileName,$tmp)) > 0 Then
         _ArrayAdd($resultArray,$fileName)
      EndIf
   WEnd
   _ArrayDelete($resultArray,0)
   _ArraySort($resultArray,0)
   return $resultArray
EndFunc

Func dirList($folder)
   Local $dirArray[1]
   Local $search = FileFindFirstFile($folder & "\*.*")
   Local $dirName
   If $search > -1 Then
      While 1
         $dirName = FileFindNextFile($search) 
         If @error Then ExitLoop
         If isDirectory($folder &"\"& $dirName) = 1 Then
            If $dirName <> "." And $dirName <> ".." Then
               _ArrayAdd($dirArray,$dirName)
            EndIf
         EndIf
      WEnd
      FileClose($search)
   EndIf
   _ArrayDelete($dirArray,0)
   _ArraySort($dirArray,0)
   return $dirArray
EndFunc

Func isDirectory($path)
   If StringInStr(FileGetAttrib($path),"D") = 0 Then
      Return 0
   Else
      Return 1 
   EndIf
EndFunc
